

class Operator(object):
    DoesNotExist = "!"
    Equals = "="
    DoubleEquals = "=="
    In = "in"
    NotEquals = "!="
    NotIn = "notin"
    Exists = "exists"
    GreaterThan = "gt"
    LessThan = "lt"
