from distutils.core import setup
from setuptools import find_packages

setup(
    name='dragonk8s',
    version='1.0.0',
    description="A k8s implement of Python3",
    author='ma',
    author_email='591867837@qq.com',
    url='https://dragonk8s.com',
    packages=find_packages(),
)